@@ -80,7 +80,7 @@ spray_negative <- function(S){
     if(is.zero(S)){
         return(S)
     } else {
-        return(spray(index(S),-value(S)))
+        return(spray(index(S),-coeffs(S)))
     }
 }
 
@@ -88,14 +88,14 @@ spray_times_spray <- function(S1,S2){
     if(is.zero(S1) || is.zero(S2)){return(spray_times_scalar(S1,0))}
     stopifnot(arity(S1) == arity(S2))
     spraymaker(spray_mult(
-        index(S1),value(S1),
-        index(S2),value(S2)
+        index(S1),coeffs(S1),
+        index(S2),coeffs(S2)
         ),arity=arity(S1))
 }
 
 spray_times_scalar <- function(S,x){
     stopifnot(length(x)==1)
-    return(spraymaker(spray(index(S), x*value(S)),arity=arity(S)))
+    return(spraymaker(spray(index(S), x*coeffs(S)),arity=arity(S)))
 }
 
 spray_plus_spray <- function(S1,S2){
@@ -107,8 +107,8 @@ spray_plus_spray <- function(S1,S2){
       }
 
     return(spraymaker(spray_add(
-    index(S1),value(S1),
-    index(S2),value(S2)
+    index(S1),coeffs(S1),
+    index(S2),coeffs(S2)
         ),arity=arity(S1)))
 }
 
@@ -121,7 +121,7 @@ spray_power_scalar <- function(S,n){
   if(n<0){
     stop("use ooom() for negative powers")
   } else {
-    return(spraymaker(spray_power(index(S),value(S),n),arity=arity(S)))
+    return(spraymaker(spray_power(index(S),coeffs(S),n),arity=arity(S)))
   }
 }
 
@@ -131,6 +131,6 @@ spray_power_scalar <- function(S,n){
     } else if(nterms(S1) != nterms(S2)){
         return(FALSE)
     } else {
-        return(spray_equality(index(S1),value(S1),index(S2),value(S2)))
+        return(spray_equality(index(S1),coeffs(S1),index(S2),coeffs(S2)))
     }
 }
